﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Diagnostics;
using System.Drawing;

using PositionChangedEventArgs = Arction.WinForms.Charting.Views.ViewXY.PositionChangedEventArgs;
using Arction.WinForms.Charting;
using Arction.WinForms.Charting.Annotations;
using Arction.WinForms.Charting.Axes;
using Arction.WinForms.Charting.EventMarkers;
using Arction.WinForms.Charting.SeriesXY;
using Arction.WinForms.Charting.Titles;
using Arction.WinForms.Charting.Views.ViewXY;


namespace AL_CursorCrosshair
{
    /// <summary>
    ///     Wrapperklasse um Arction-Linien zur Vereinfachung der Handhabung
    /// </summary>
    public class CrosshairCursor
    {
        #region Membervariablen

        protected int _cursorIndex = -1;
        protected ViewXY _view;
        protected AxisX _axisX;
        protected AxisY _axisY;

        protected LineSeriesCursor _cursorVertical = null;
        protected ConstantLine _cursorHorizontal = null;
        protected AnnotationXY _cursorCenter = null;

        protected double _x = double.NaN;
        protected double _y = double.NaN;

        #endregion
        #region Properties

        public double X
        {
            get { return _x; }
            set
            {
                if (value.EEquals(_x)) return;
                SetPositions(value, _y);
            }
        }
        public double Y
        {
            get { return _y; }
            set
            {
                if (value.EEquals(_y)) return;
                SetPositions(_x, value);
            }
        }

        #endregion
        #region Events

        private bool _suppressEvent = false;
        public event EventHandler<CursorEventArgs> CursorMoved;

        protected virtual void FireCursorMoved()
        {
            if (_suppressEvent) return;

            if (CursorMoved != null)
            {
                CursorMoved(this, new CursorEventArgs() { CursorIndex = _cursorIndex, X = _x, Y = _y });
            }
        }

        #endregion

        #region Constructor

        public CrosshairCursor(int cursorIndex, ViewXY view, AxisX axisX, AxisY axisY)
        {
            _cursorIndex = cursorIndex;
            _view = view;
            _axisX = axisX;
            _axisY = axisY;

            {
                _cursorVertical = new LineSeriesCursor(_view, _axisX)
                {
                    Behind = false,
                    FullHeight = false,
                    HairCrossColorMixing = false,
                    LineStyle = {Pattern = LinePattern.Dot, AntiAliasing = LineAntialias.None, Color = Color.Black, Width = 1},
                    MouseHighlight = MouseOverHighlight.None,
                    MouseInteraction = true,
                    MoveByMouse = true,
                    Visible = true,
                };
                _cursorVertical.Style = CursorStyle.VerticalNoTracking;
                _cursorVertical.PositionChanged += CursorVerticalPositionChanged;
                _view.LineSeriesCursors.Add(_cursorVertical);
            }

            {
                _cursorHorizontal = new ConstantLine(_view, axisX, axisY)
                {
                    Behind = false,
                    LineStyle = { Pattern = LinePattern.Dot, AntiAliasing = LineAntialias.None, Color = Color.Black, Width = 1 },
                    MouseHighlight = MouseOverHighlight.None,
                    MouseInteraction = true,
                    MoveByMouse = true,
                    ShowInLegendBox = false,
                    IncludeInAutoFit = false,
                    Visible = true,
                    Value = 0,
                    LimitYToStackSegment = false,
                };
                _cursorHorizontal.ValueChanged += CursorHorizontalOnValueChanged;
                _view.ConstantLines.Add(_cursorHorizontal);
            }

            {
                _cursorCenter = new AnnotationXY(_view, axisX, axisY)
                {
                    Style = AnnotationStyle.Rectangle,
                    Sizing = AnnotationXYSizing.ScreenCoordinates,
                    BorderVisible = false,
                    Text = string.Empty,
                    Behind = false,
                    Visible = true,
                    //KeepVisible = true,

                    //LocationCoordinateSystem = CoordinateSystem.RelativeCoordinatesToTarget,
                    LocationCoordinateSystem = CoordinateSystem.AxisValues,
                    
                    TargetCoordinateSystem = AnnotationTargetCoordinates.AxisValues,

                    AnchorAdjustByMouse = false,
                    ResizeByMouse = false,
                    RotateByMouse = false,
                    MouseHighlight = MouseOverHighlight.None,
                };
                _cursorCenter.Shadow.Visible = false;
                _cursorCenter.Anchor.X = 0.0f;
                _cursorCenter.Anchor.Y = 0.0f;
                _cursorCenter.Fill.Style = RectFillStyle.ColorOnly;
                _cursorCenter.Fill.GradientFill = GradientFill.Solid;

                // Hide the rectangle, red for DEBUG
                //_cursorCenter.Fill.Color = Color.Transparent;
                _cursorCenter.Fill.Color = Color.FromArgb(128, Color.Red);

                _cursorCenter.SizeScreenCoords.Width = 32;
                _cursorCenter.SizeScreenCoords.Height = 32;
                _cursorCenter.LocationRelativeOffset.X = -16;
                _cursorCenter.LocationRelativeOffset.Y = -16;

                _cursorCenter.LocationCoordinateSystem = CoordinateSystem.AxisValues;

                // Hide the target nibble:
                _cursorCenter.NibStyle.SizeFactor = -1;
                _cursorCenter.NibStyle.Alpha = 0;

                _view.Annotations.Add(_cursorCenter);

                // Events
                //_cursorCenter.LocationRelativeOffsetChangedByMouse += CursorOnPositionChanged;
                _cursorCenter.LocationAxisValuesChangedByMouse += CursorCenterOnLocationAxisValuesChangedByMouse;
            }
        }

        #endregion

        public virtual void SetPositions(double x, double y)
        {
            _x = x;
            _y = y;

            {
                _cursorVertical.Visible = true;
                _cursorVertical.ValueAtXAxis = x;
            }

            if (_cursorHorizontal != null)
            {
                _cursorHorizontal.Visible = true;
                _cursorHorizontal.Value = y;
            }
            if (_cursorCenter != null)
            {
                _cursorCenter.TargetAxisValues.X = _x;
                _cursorCenter.TargetAxisValues.Y = _y;
            }
            UpdateCenter();
        }

        protected void UpdateCenter()
        {
            if (_cursorCenter != null)
            {
                _cursorCenter.TargetAxisValues.X = _x;
                _cursorCenter.TargetAxisValues.Y = _y;
                _cursorCenter.LocationAxisValues.X = _x;
                _cursorCenter.LocationAxisValues.Y = _y;
            }
        }

        #region Eventhandler

        private void CursorVerticalPositionChanged(object sender, PositionChangedEventArgs e)
        {
            _x = e.NewValue;
            _suppressEvent = true;
            UpdateCenter();
            _suppressEvent = false;
            FireCursorMoved();
        }

        private void CursorHorizontalOnValueChanged(object sender, ValueChangedEventArgs e)
        {
            _y = e.NewValue;
            _suppressEvent = true;
            UpdateCenter();
            _suppressEvent = false;
            FireCursorMoved();
        }

        /*
        private void CursorOnPositionChanged(object sender, LocationRelativeOffsetChangedByMouseEventArgs e)
        {
            _suppressEvent = true;
            _x = e.X;
            _y = e.Y;
            e.CancelRendering = true;
            _cursorCenter.LocationRelativeOffset.X = -16;
            _cursorCenter.LocationRelativeOffset.Y = -16;
            SetPositions(_x, _y);
            _suppressEvent = false;

            FireCursorMoved();
        }
        */

        private void CursorCenterOnLocationAxisValuesChangedByMouse(object sender, LocationAxisValuesChangedByMouseXYEventArgs e)
        {
            _suppressEvent = true;
            _x = e.XValue;
            _y = e.YValue;
            SetPositions(_x, _y);
            _suppressEvent = false;

            FireCursorMoved();

        }

        #endregion
    }

}

